package com.jz.dmp.modules.controller;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.dmp.modules.controller.bean.DmpNavigationTreeDto;
import com.jz.dmp.modules.controller.bean.DmpNavigationTreeRequest;
import com.jz.dmp.modules.model.DmpNavigationTree;
import com.jz.dmp.modules.service.DmpNavigationTreeService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * DMP资源导航树(DmpNavigationTree)表控制层
 *
 * @author Bellamy
 * @since 2020-12-29 15:08:17
 */
@RestController
@RequestMapping("/dmpNavigationTree")
@Api(tags = "DMP资源导航树")
public class DmpNavigationTreeController {
    /**
     * 服务对象
     */
    @Autowired
    private DmpNavigationTreeService dmpNavigationTreeService;

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @GetMapping("/selectOne")
    @ApiOperation(value = "通过主键查询单条数据", notes = "通过主键查询单条数据")
    public DmpNavigationTree selectOne(Integer id) {
        return this.dmpNavigationTreeService.queryById(id);
    }
    
    /**列表查询DMP资源导航树
	 * @param dmpNavigationTreeRequest
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/findList")
	@ApiOperation(value = "列表查询DMP资源导航树", notes = "列表查询DMP资源导航树")
	public BaseBeanResponse<DmpNavigationTreeDto> findList(@RequestBody DmpNavigationTreeRequest dmpNavigationTreeRequest, HttpServletRequest httpRequest){
		BaseBeanResponse<DmpNavigationTreeDto> baseBeanResponse = new BaseBeanResponse<DmpNavigationTreeDto>();
		try {
			baseBeanResponse = dmpNavigationTreeService.findList(dmpNavigationTreeRequest, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("查询失败");
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}
	
	/**新增DMP资源导航树
	 * @param dmpNavigationTreeRequest
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/add")
	@ApiOperation(value = "新增DMP资源导航树", notes = "新增DMP资源导航树")
	public BaseBeanResponse<DmpNavigationTree> add(@RequestBody DmpNavigationTree dmpNavigationTree, HttpServletRequest httpRequest){
		BaseBeanResponse<DmpNavigationTree> baseBeanResponse = new BaseBeanResponse<DmpNavigationTree>();
		try {
			DmpNavigationTree dmpNavigationTreeDb = dmpNavigationTreeService.insert(dmpNavigationTree);
			baseBeanResponse.setCode(StatuConstant.CODE_SUCCESS);
			baseBeanResponse.setMessage("新增成功");
			baseBeanResponse.setData(dmpNavigationTreeDb);
		} catch (Exception e) {
			baseBeanResponse.setMessage("新增失败");
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}

}