package com.jz.dmp.modules.controller.sys;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.dmp.modules.controller.sys.bean.DmpPermissionBatch;
import com.jz.dmp.modules.controller.sys.bean.DmpPermissionDto;
import com.jz.dmp.modules.controller.sys.bean.DmpPermissionRequest;
import com.jz.dmp.modules.model.DmpPermission;
import com.jz.dmp.modules.service.sys.DmpPermissionService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 资源表Controller
 * @author ybz
 *
 */
@RestController
@RequestMapping("/dmp/permission")
@Api(tags = "资源表")
public class DmpPermissionController {
	
	@Autowired
	private DmpPermissionService dmpPermissionService;
	
	/**列表查询资源表
	 * @param dmpPermissionRequest
	 * @return
	 */
	//@RequestMapping(method = RequestMethod.POST, value = "/findList")
	//@ApiOperation(value = "列表查询资源表", notes = "列表查询资源表")
	public PageInfoResponse<DmpPermissionDto> findList(@RequestBody DmpPermissionRequest dmpPermissionRequest, HttpServletRequest httpRequest){
		PageInfoResponse<DmpPermissionDto> pageInfo = new PageInfoResponse<DmpPermissionDto>();
		try {
			pageInfo = dmpPermissionService.findList(dmpPermissionRequest, httpRequest);
		} catch (Exception e) {
			pageInfo.setMessage("查询失败");
			pageInfo.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return pageInfo;
	}
	
	
	/**主键查询资源表
	 * @param dmpPermissionRequest
	 * @return
	 */
	//@RequestMapping(method = RequestMethod.POST, value = "/view")
	//@ApiOperation(value = "主键查询资源表", notes = "主键查询资源表")
	public BaseBeanResponse<DmpPermissionDto> view(@RequestBody DmpPermissionRequest dmpPermissionRequest, HttpServletRequest httpRequest){
		BaseBeanResponse<DmpPermissionDto> baseBeanResponse = new BaseBeanResponse<DmpPermissionDto>();
		try {
			baseBeanResponse = dmpPermissionService.findById(dmpPermissionRequest, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("请求失败");
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}
	
	/**删除资源表
	 * @param dmpPermissionRequest
	 * @return
	 */
	//@RequestMapping(method = RequestMethod.POST, value = "/delete")
	//@ApiOperation(value = "删除资源表", notes = "删除资源表")
	public BaseResponse delete(@RequestBody DmpPermissionRequest dmpPermissionRequest, HttpServletRequest httpRequest){
		BaseResponse baseResponse = new BaseResponse();
		try {
			baseResponse = dmpPermissionService.deleteById(dmpPermissionRequest, httpRequest);
		} catch (Exception e) {
			baseResponse.setMessage("删除失败");
			baseResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseResponse;
	}
	
	/**新增资源表
	 * @param dmpPermissionRequest
	 * @return
	 */
	//@RequestMapping(method = RequestMethod.POST, value = "/add")
	//@ApiOperation(value = "新增资源表", notes = "新增资源表")
	public BaseBeanResponse<DmpPermission> add(@RequestBody DmpPermission dmpPermission, HttpServletRequest httpRequest){
		BaseBeanResponse<DmpPermission> baseBeanResponse = new BaseBeanResponse<DmpPermission>();
		try {
			baseBeanResponse = dmpPermissionService.add(dmpPermission, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("新增失败");
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}
	
	/**批量新增资源表
	 * @param dmpPermissionRequest
	 * @return
	 */
	//@RequestMapping(method = RequestMethod.POST, value = "/addBatch")
	//@ApiOperation(value = "批量新增资源表", notes = "批量新增资源表")
	public BaseBeanResponse<DmpPermissionBatch> addBatch(@RequestBody DmpPermissionBatch dmpPermissionBatch, HttpServletRequest httpRequest){
		BaseBeanResponse<DmpPermissionBatch> baseBeanResponse = new BaseBeanResponse<DmpPermissionBatch>();
		try {
			baseBeanResponse = dmpPermissionService.addBatch(dmpPermissionBatch, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("批量新增失败");
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}
	
	/**修改资源表
	 * @param dmpPermissionRequest
	 * @return
	 */
	//@RequestMapping(method = RequestMethod.POST, value = "/edit")
	//@ApiOperation(value = "修改资源表", notes = "修改资源表")
	public BaseBeanResponse<DmpPermission> edit(@RequestBody DmpPermission dmpPermission, HttpServletRequest httpRequest){
		BaseBeanResponse<DmpPermission> baseBeanResponse = new BaseBeanResponse<DmpPermission>();
		try {
			baseBeanResponse = dmpPermissionService.edit(dmpPermission, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("修改失败");
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}
	
	/**根据父级资源编码，获取当前登录用户所有资源
	 * @param dmpPermissionRequest
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/findListByParentCode")
	@ApiOperation(value = "获取当前用户指定资源", notes = "根据父级资源编码，获取当前登录用户所有资源")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "parentCode", value = "父资源编码"),
		@ApiImplicitParam(name = "type", value = "请求类型：0：用户项目下资源；1：用户系统级别资源")
	})
	public BaseBeanResponse<DmpPermissionDto> findListByParentCode(@RequestParam(name = "parentCode", required = true) String parentCode, @RequestParam(name = "type", required = true) String type, HttpServletRequest httpRequest){
		BaseBeanResponse<DmpPermissionDto> baseBeanResponse = new BaseBeanResponse<DmpPermissionDto>();
		
		try {
			baseBeanResponse = dmpPermissionService.findListByParentCode(parentCode, type, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("查询失败");
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}
	
}
