package com.jz.dmp.modules.service.impl;

import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.common.persistence.BaseService;
import com.jz.dmp.modules.controller.bean.DmpNavigationTreeDto;
import com.jz.dmp.modules.controller.bean.DmpNavigationTreeRequest;
import com.jz.dmp.modules.controller.bean.MyDmpNavigationTreeConverter;
import com.jz.dmp.modules.dao.DmpNavigationTreeDao;
import com.jz.dmp.modules.model.DmpNavigationTree;
import com.jz.dmp.modules.service.DmpNavigationTreeService;

/**
 * DMP资源导航树(DmpNavigationTree)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-29 15:08:16
 */
@Service("dmpNavigationTreeService")
public class DmpNavigationTreeServiceImpl extends BaseService implements DmpNavigationTreeService {

    @Autowired
    private DmpNavigationTreeDao dmpNavigationTreeDao;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public DmpNavigationTree queryById(Integer id) {
        return this.dmpNavigationTreeDao.queryById(id);
    }

    /**
     * 查询多条数据
     *
     * @param offset 查询起始位置
     * @param limit  查询条数
     * @return 对象列表
     */
    @Override
    public List<DmpNavigationTree> queryAllByLimit(int offset, int limit) {
        return this.dmpNavigationTreeDao.queryAllByLimit(offset, limit);
    }

    /**
     * 新增数据
     *
     * @param dmpNavigationTree 实例对象
     * @return 实例对象
     * @throws Exception 
     */
    @Override
    public DmpNavigationTree insert(DmpNavigationTree dmpNavigationTree) throws Exception {
		// 创建人
		dmpNavigationTree.setCreateUserId(getHttpRequestUserId(null).toString());
		// 创建时间
		dmpNavigationTree.setCreateTime(new Date());
        this.dmpNavigationTreeDao.insert(dmpNavigationTree);
        return dmpNavigationTree;
    }

    /**
     * 修改数据
     *
     * @param dmpNavigationTree 实例对象
     * @return 实例对象
     */
    @Override
    public DmpNavigationTree update(DmpNavigationTree dmpNavigationTree) {
        this.dmpNavigationTreeDao.update(dmpNavigationTree);
        return this.queryById(dmpNavigationTree.getId());
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @Override
    public boolean deleteById(Integer id) {
        return this.dmpNavigationTreeDao.deleteById(id) > 0;
    }
    
    /*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpNavigationTreeService#findList(com.ycxc.vmts.controller.bas.
	 * bean.DmpNavigationTreeRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpNavigationTreeDto> findList(DmpNavigationTreeRequest dmpNavigationTreeRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpNavigationTreeDto> baseBeanResponse = new BaseBeanResponse<>();

		DmpNavigationTree dmpNavigationTree = new DmpNavigationTree();
		//ID
		if (dmpNavigationTreeRequest.getId() != null) {
			dmpNavigationTree.setId(dmpNavigationTreeRequest.getId());
		}
		//项目ID
		if (dmpNavigationTreeRequest.getProjectId() != null) {
			dmpNavigationTree.setProjectId(dmpNavigationTreeRequest.getProjectId());
		}
		//树类别
		if (!StringUtils.isEmpty(dmpNavigationTreeRequest.getCategory())) {
			dmpNavigationTree.setCategory(dmpNavigationTreeRequest.getCategory());
		}
		//树类型
		if (!StringUtils.isEmpty(dmpNavigationTreeRequest.getType())) {
			dmpNavigationTree.setType(dmpNavigationTreeRequest.getType());
		}
		//名称
		if (!StringUtils.isEmpty(dmpNavigationTreeRequest.getName())) {
			dmpNavigationTree.setName(dmpNavigationTreeRequest.getName());
		}
		//序号
		if (dmpNavigationTreeRequest.getTreeSort() != null) {
			dmpNavigationTree.setTreeSort(dmpNavigationTreeRequest.getTreeSort());
		}
		//是否叶子节点(0:否，1:是)
		if (!StringUtils.isEmpty(dmpNavigationTreeRequest.getIsLevel())) {
			dmpNavigationTree.setIsLevel(dmpNavigationTreeRequest.getIsLevel());
		}
		//是否启用(0:否，1:是)
		if (!StringUtils.isEmpty(dmpNavigationTreeRequest.getIsEnable())) {
			dmpNavigationTree.setIsEnable(dmpNavigationTreeRequest.getIsEnable());
		}
		//父节点ID
		if (dmpNavigationTreeRequest.getParentId() != null) {
			dmpNavigationTree.setParentId(dmpNavigationTreeRequest.getParentId());
		}
		//数据状态
		if (!StringUtils.isEmpty(dmpNavigationTreeRequest.getDataStatus())) {
			dmpNavigationTree.setDataStatus(dmpNavigationTreeRequest.getDataStatus());
		}
		//创建用户ID
		if (!StringUtils.isEmpty(dmpNavigationTreeRequest.getCreateUserId())) {
			dmpNavigationTree.setCreateUserId(dmpNavigationTreeRequest.getCreateUserId());
		}
		//创建用户ID
		if (!StringUtils.isEmpty(dmpNavigationTreeRequest.getUpdateUserId())) {
			dmpNavigationTree.setUpdateUserId(dmpNavigationTreeRequest.getUpdateUserId());
		}

		List<DmpNavigationTree> list = dmpNavigationTreeDao.queryAll(dmpNavigationTree);
		
		List<DmpNavigationTreeDto> dtos = MyDmpNavigationTreeConverter.INSTANCE().domain2dto(list);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setDatas(dtos);
		return baseBeanResponse;
	}
}