package com.jz.dmp.modules.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.agent.service.DmpDsAgentService;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.enums.TestConnectStatusEnum;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.JsonMapper;
import com.jz.common.utils.web.SessionUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.modules.controller.DataIntegration.bean.DataSourceListDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.DataSourceListReq;
import com.jz.dmp.modules.controller.DataIntegration.bean.DmpSyncingDatasourceReq;
import com.jz.dmp.modules.dao.DmpProjectDao;
import com.jz.dmp.modules.dao.DmpSyncingDatasourceDao;
import com.jz.dmp.modules.model.DmpAgentDatasourceInfo;
import com.jz.dmp.modules.model.DmpProjectSystemInfo;
import com.jz.dmp.modules.model.DmpSyncingDatasource;
import com.jz.dmp.modules.model.DmpSyncingDatasourceType;
import com.jz.dmp.modules.service.DmpSyncingDatasourceService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * 数据源配置(DmpSyncingDatasource)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-24 14:09:59
 */
@Service("dmpSyncingDatasourceService")
@Transactional
public class DmpSyncingDatasourceServiceImpl implements DmpSyncingDatasourceService {

    private static Logger logger = LoggerFactory.getLogger(DmpSyncingDatasourceServiceImpl.class);

    @Value("${spring.public-key}")
    private String publicKey;

    @Autowired
    private DmpSyncingDatasourceDao dmpSyncingDatasourceDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    @Autowired
    private DmpDsAgentService dmpDsAgentServiceImp;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public DmpSyncingDatasource queryById(Integer id) {
        return this.dmpSyncingDatasourceDao.queryById(id);
    }

    /**
     * 查询多条数据
     *
     * @param offset 查询起始位置
     * @param limit  查询条数
     * @return 对象列表
     */
    @Override
    public List<DmpSyncingDatasource> queryAllByLimit(int offset, int limit) {
        return this.dmpSyncingDatasourceDao.queryAllByLimit(offset, limit);
    }

    /**
     * 新增数据
     *
     * @param dmpSyncingDatasource 实例对象
     * @return 实例对象
     * @author Bellamy
     */
    @Override
    public DmpSyncingDatasource insert(DmpSyncingDatasource dmpSyncingDatasource) {
        this.dmpSyncingDatasourceDao.insert(dmpSyncingDatasource);
        return dmpSyncingDatasource;
    }

    /**
     * 修改数据
     *
     * @param dmpSyncingDatasource 实例对象
     * @return 实例对象
     * @author Bellamy
     */
    @Override
    public DmpSyncingDatasource update(DmpSyncingDatasource dmpSyncingDatasource) {
        this.dmpSyncingDatasourceDao.update(dmpSyncingDatasource);
        return this.queryById(dmpSyncingDatasource.getId());
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     * @author Bellamy
     */
    @Override
    public boolean deleteById(Integer id) {
        return this.dmpSyncingDatasourceDao.deleteById(id) > 0;
    }

    /**
     * 数据源列表查询
     *
     * @param req
     * @author Bellamy
     */
    @Override
    public PageInfoResponse<DataSourceListDto> queryDataSourceListPage(DataSourceListReq req, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<DataSourceListDto> pageInfoResponse = new PageInfoResponse<>();

        PageHelper.startPage(req.getPageNum(), req.getPageSize());
        req.setDatasourceName(req.getDatasourceName().trim());
        List<DataSourceListDto> list = dmpSyncingDatasourceDao.queryDataSourceListPage(req);
        PageInfo<DataSourceListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 批量删除数据源
     *
     * @return
     * @author Bellamy
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public JsonResult delDataSourceById(String datasourceId) throws Exception {
        String[] ids = datasourceId.split(",");
        dmpSyncingDatasourceDao.delDataSourceById(ids);
        return new JsonResult();
    }

    /**
     * 获取数据源类型-下拉框
     *
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult queryDatasourceType() throws Exception {
        List<Map> list = dmpSyncingDatasourceDao.queryDatasourceType();
        return new JsonResult(list);
    }

    /**
     * 新增-获取数据源类型
     *
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult queryGroupDatasourceType() throws Exception {
        List<Map> returnList = new ArrayList<>();
        Map map = new HashMap();
        List<Map> list = dmpSyncingDatasourceDao.queryGroupDatasourceType(map);
        List<Map> typeList = dmpSyncingDatasourceDao.queryDbTypeByGroup();
        if (list.size() > 0 && list != null) {
            for (Map typeDto : typeList) {
                Map<String, Object> returnMap = new LinkedHashMap();
                String datasourceCatecode = typeDto.get("datasourceCatecode").toString();
                returnMap.put("dbValue", new ArrayList<Map>());
                for (Map<String, Object> dto : list) {
                    if (datasourceCatecode.equals(dto.get("datasourceCatecode"))) {
                        returnMap.put("typeName", dto.get("datasourceCatename"));
                        List<Map> param = (List<Map>) returnMap.get("dbValue");
                        param.add(dto);
                    }
                }
                returnList.add(returnMap);
            }
        }
        return new JsonResult(returnList);
    }

    /**
     * 保存数据源
     *
     * @return
     * @author Bellamy
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    @Override
    public JsonResult addDatasourceInfo(DmpSyncingDatasourceReq saveBody) throws Exception {
        if (StringUtils.isEmpty(saveBody.getDatasourceName())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "数据源名称不能为空");
        }
        if (StringUtils.isEmpty(saveBody.getDatasourceType())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "数据源类型ID不能为空");
        }
        if (StringUtils.isEmpty(saveBody.getProjectId())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "项目ID不能为空");
        }
        //通过名称查询数据源是否存在
        int len = dmpSyncingDatasourceDao.countDatasourceByName(saveBody.getDatasourceName(), saveBody.getProjectId());
        if (len > 0) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "数据源名称已存在");
        }

        DmpSyncingDatasource dsd = new DmpSyncingDatasource();
        BeanUtils.copyProperties(saveBody, dsd);

        dsd.setCreateTime(new Date());
        dsd.setCreateUserId(SessionUtils.getCurrentUserId());
        dsd.setDatasourceType(Integer.valueOf(saveBody.getDatasourceType())); //数据源类型ID
        dsd.setProjectId(Integer.valueOf(saveBody.getProjectId()));
        dsd.setDataStatus("1");
        dsd.setTestConnectStatus(saveBody.getTestConnectStatus());
        if (StringUtils.isEmpty(saveBody.getTestConnectStatus())) { //默认未测试
            dsd.setTestConnectStatus(TestConnectStatusEnum.WCS.getValue());
        }

        //解析要保存的数据源信息
        DmpSyncingDatasource dmpSyncingDatasource = getDataSourceInfo(saveBody);
        dsd.setPassword(dmpSyncingDatasource.getPassword()); //密码
        if (StringUtils.isNotEmpty(dmpSyncingDatasource.getDbName())) {
            dsd.setDbName(dmpSyncingDatasource.getDbName()); //数据库名称
        }
        if (StringUtils.isNotEmpty(dmpSyncingDatasource.getHost())) {
            dsd.setHost(dmpSyncingDatasource.getHost()); //IP
        }
        if (StringUtils.isNotEmpty(dmpSyncingDatasource.getPort())) {
            dsd.setPort(dmpSyncingDatasource.getPort()); //端口
        }
        dmpSyncingDatasourceDao.insert(dsd);
        return new JsonResult();
    }

    /**
     * 解析要保存的数据源信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    public DmpSyncingDatasource getDataSourceInfo(DmpSyncingDatasourceReq saveBody) {
        DmpSyncingDatasource returnData = new DmpSyncingDatasource();

        if (saveBody != null) {
            String jdbcUrl = saveBody.getJdbcUrl();
            if (!org.springframework.util.StringUtils.isEmpty(jdbcUrl)) {
                jdbcUrl = jdbcUrl.substring(jdbcUrl.indexOf("//") + 2);
                String hostPort = "";
                String dbName = "";
                if (jdbcUrl.contains("/")) {
                    hostPort = jdbcUrl.substring(0, jdbcUrl.indexOf("/"));
                    dbName = jdbcUrl.substring(jdbcUrl.indexOf("/") + 1);
                }
                if (dbName.contains("?")) {
                    dbName = dbName.substring(0, dbName.indexOf("?")); //数据库名称
                }
                returnData.setDbName(dbName);
                String[] hostPortArr = hostPort.split(":");
                if (hostPortArr != null && hostPortArr.length >= 2) {
                    String host = hostPortArr[0];
                    String port = hostPortArr[1];
                    returnData.setHost(host); //ip
                    returnData.setPort(port); //端口
                }

            }
            //加密密码
            String password = saveBody.getPassword();
            password = new BaseService().encode(password, publicKey);
            returnData.setPassword(password);
        }
        return returnData;
    }

    /**
     * 测试连通性
     *
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult testConnection(DmpSyncingDatasourceReq saveBody) throws Exception {
        DmpAgentDatasourceInfo ds = this.dsInfoDTO(saveBody); //查询数据源 对应的 数据库信息
        DmpAgentResult rst = dmpDsAgentServiceImp.testConnect(ds); //连接测试
        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        } else {
            //连接测试成功
            rst.setResult(JsonMapper.fromJsonString(rst.getMessage(), Boolean.class));
            return new JsonResult(ResultCode.SUCCESS, rst);
        }
    }

    /**
     * 编辑数据源，根据id查询数据回显
     *
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult selectDataSourceInfoById(Map map) throws Exception {
        DataSourceListDto asd = dmpSyncingDatasourceDao.selectDataSourceInfoById(map);
        return new JsonResult(asd);
    }

    /**
     * 编辑数据源
     *
     * @return
     * @author Bellamy
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult updateDatasourceById(DmpSyncingDatasourceReq saveBody) throws Exception {
        DmpSyncingDatasource dsd = new DmpSyncingDatasource();
        BeanUtils.copyProperties(saveBody, dsd);

        dsd.setUpdateTime(new Date());
        dsd.setUpdateUserId(SessionUtils.getCurrentUserId());
        dsd.setDataStatus("1");
        if (StringUtils.isNotEmpty(saveBody.getDatasourceType())) {
            dsd.setDatasourceType(Integer.valueOf(saveBody.getDatasourceType())); //数据源类型ID
        }
        if (StringUtils.isNotEmpty(saveBody.getProjectId())) {
            dsd.setProjectId(Integer.valueOf(saveBody.getProjectId()));
        }
        if (StringUtils.isNotEmpty(saveBody.getDatasourceId())) { //数据源id
            dsd.setId(Integer.valueOf(saveBody.getDatasourceId()));
        }
        dsd.setTestConnectStatus(saveBody.getTestConnectStatus());
        if (StringUtils.isEmpty(saveBody.getTestConnectStatus())) { //默认未测试
            dsd.setTestConnectStatus(TestConnectStatusEnum.WCS.getValue());
        }
        //解析要保存的数据源信息
        DmpSyncingDatasource dmpSyncingDatasource = getDataSourceInfo(saveBody);
        dsd.setPassword(dmpSyncingDatasource.getPassword()); //密码
        if (StringUtils.isNotEmpty(dmpSyncingDatasource.getDbName())) {
            dsd.setDbName(dmpSyncingDatasource.getDbName()); //数据库名称
        }
        if (StringUtils.isNotEmpty(dmpSyncingDatasource.getHost())) {
            dsd.setHost(dmpSyncingDatasource.getHost()); //IP
        }
        if (StringUtils.isNotEmpty(dmpSyncingDatasource.getPort())) {
            dsd.setPort(dmpSyncingDatasource.getPort()); //端口
        }

        int len = dmpSyncingDatasourceDao.update(dsd);
        if (len > 0) {
            return new JsonResult();
        } else {
            return new JsonResult(ResultCode.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public List<DmpSyncingDatasource> findListByParams(DmpSyncingDatasource ds) throws Exception {
        return dmpSyncingDatasourceDao.findListByParams(ds);
    }

    /**
     * 获取数据源类型输入框属性
     *
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult selectDatasourceTypeAttrById(String datasourceTypeId) throws Exception {
        Map returnMap = new HashMap();
        Map map = new HashMap();
        map.put("datasourceTypeId", datasourceTypeId);
        List<Map> list = dmpSyncingDatasourceDao.queryGroupDatasourceType(map);
        if (CollectionUtils.isNotEmpty(list)) {
            Map str = list.get(0);
            returnMap = JSONObject.parseObject(str.get("dbAttrs").toString());
        }
        return new JsonResult(returnMap);
    }

    /*
     * 查询数据源 对应的 数据库信息
     * @author Bellamy
     * */
    private DmpAgentDatasourceInfo dsInfoDTO(DmpSyncingDatasourceReq body) throws Exception {
        //数据源类型ID去查询
        DmpSyncingDatasourceType type = dmpSyncingDatasourceDao.queryDatasourceTypeById(body.getDatasourceType());
        DmpAgentDatasourceInfo ds = new DmpAgentDatasourceInfo();
        ds.setDatasourceType(type.getDatasourceType());
        ds.setDatasourceName(body.getDatasourceName());
        ds.setDbName(body.getDbName());
        ds.setJdbcUrl(body.getJdbcUrl());
        ds.setUserName(body.getUserName());
        ds.setPassword(body.getPassword());
        ds.setDriverClassName(type.getDriverClassName());
        ds.setHost(body.getHost());
        ds.setPort(body.getPort());
        ds.setDelimiter(body.getDelimiter());
        ds.setProtocol(body.getProtocol());
        ds.setProjectId(body.getProjectId() + "");
        ds.setAccessId(body.getAccessId());
        ds.setAccessKey(body.getAccessKey());
        ds.setIsHaveHeader(body.getIsHaveHeader());
        ds.setEndpoint(body.getEndpoint());
        ds.setDefaultFs(body.getDefaultFs());

        DmpProjectSystemInfo info = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(body.getProjectId()));
        ds.setKerberosIsenable(info.getKerberosIsenable());
        ds.setKerberosJaasConf(info.getKerberosJaasConf());
        ds.setKerberosKrb5Conf(info.getKerberosKrb5Conf());
        ds.setKerberosFqdn(info.getKerberosFqdn());
        ds.setHdfsDefaultConf(""); // info.getHdfsDefaultConf());
        ds.setHdfsConfPath(""); // info.getHdfsConfPath());
        ds.setHdfsAuthPath(body.getAccessKey());
        ds.setHdfsUserName(info.getHdfsUserName());
        ds.setHdfsSyncingPath(info.getHdfsSyncingPath());
        return ds;
    }
}